/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005-2006 Continuent, Inc.
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Gilles Rayrat
 * Contributor(s): 
 */

#ifndef CONNECTIONPARAMETERS_H_
#define CONNECTIONPARAMETERS_H_

#include "ControllerPool.hpp"
#include "CarobException.hpp"

#ifdef __MINGW32__
  #include <winsock2.h>
  #include <ws2tcpip.h>
  #include "Common.hpp" //for in_port_t
#else
  #include <netinet/in.h> // in_port_t
#endif
#include <string>
#include <vector>
namespace CarobNS {

/**
 * Identifies a controller.
 * For now a controller is identified by an hostname and a port.
 * TODO: In the future, only its sockaddr should be kept (see CAROB-61)
 */
class ControllerInfo
{
public:
  /** Empty constructor to set default values (host="" and port=0) */
  ControllerInfo();
  /** Copy contructor */
  ControllerInfo(const ControllerInfo& ci);
  /** Constructs a Controller info with the given name and port */
  ControllerInfo(const std::wstring& host, in_port_t port);
  virtual ~ControllerInfo() {}
  /** Copy operator */
  ControllerInfo& operator =(const ControllerInfo& ci);
  /** Comparison operator */
  bool operator ==(const ControllerInfo& ci) const;
  /**
   * Utility lesser than operator for ControllerPoolManager
   * For performance purposes, the comparison is done first on host_port, then
   * on host_name
   */
  bool operator <(const ControllerInfo& ci) const;
  /**
   * Utility greater than operator for ControllerPoolManager
   * For performance purposes, the comparison is done first on host_port, then
   * on host_name
   */
  bool operator >(const ControllerInfo& ci) const;
  /** Gives the host name */
  std::wstring  getHostName() const { return host_name; }
  /** Gives the host port */
  in_port_t     getHostPort() const { return host_port; }
  /**
   * Cast to wstring operator to ease printing (for debug/traces).
   * @return sql query as a string
   */
  operator      std::wstring () const;
private:
  /** String representation of the host name, fully qualified or IP address */
  std::wstring  host_name;
  /** Host port */
  in_port_t     host_port;
};

/**
 * This class contains the parameters for a connection to the controller.
 * This connection parameters will be used by the Connection class.
 * It is a replacement for SequoiaUrl Java class
 */
class ConnectionParameters
{
public:
  /** Default username = "user" */
  static const std::wstring    DEFAULT_USER;
  /** Default password = "" */
  static const std::wstring    DEFAULT_PASSWD;
  /** Default database name = "myDB" */
  static const std::wstring    DEFAULT_DB;
  /** Default policy = ROUND_ROBIN */
  static const ConnectPolicy   DEFAULT_POLICY;
  /** Default for persistent connections value = false */
  static const bool            DEFAULT_CONNECTION_PERSISTENCY;
  /** Default for retrieve SQL warnings = false */
  static const bool            DEFAULT_RETRIEVE_SQL_WARNINGS;

  /**
   * Creates instance with a set of values:
   * @param ctrls a list of controllers to connect to as a vector of
   *        ControllerInfo.
   * @param db name of the database (default to "user)
   * @param uname user name to use on this database (default to "")
   * @param upass user pass (default to "myDB")
   * @param cp controller pool policy for failover, one of enum ConnectPolicy
   *        (default to ROUND_ROBIN)
   * @param persistentConnection whether to create a persistent connection
   *        (default to false)
   * @param retrieveSQLWarnings whether SQL Warnings should be retrieved
   *        (default to false)
   */
  ConnectionParameters(const std::vector<ControllerInfo>& ctrls,
                       const std::wstring&  db                = DEFAULT_DB,
                       const std::wstring&  uname             = DEFAULT_USER,
                       const std::wstring&  upass             = DEFAULT_PASSWD,
                       ConnectPolicy        cp                = DEFAULT_POLICY,
                       bool                 persistentConnection = DEFAULT_CONNECTION_PERSISTENCY,
                       bool                 retrieveSQLWarnings = DEFAULT_RETRIEVE_SQL_WARNINGS)
                   throw (ConnectionException, UnexpectedException);
	virtual ~ConnectionParameters();
  /**
   * Validates database name. If invalid returns false and set param char to the
   * first invalid character.
   */
  bool                isValidDatabaseName(const std::wstring&, wchar_t&) const;
  /**
   * Validates host name. If invalid returns false and set param char to the
   * first invalid character.
   */
  bool                isValidHostName(const std::wstring&, wchar_t&) const;

  /** Checks the host name */
  void                checkHostName(const std::wstring&) const
                          throw (ConnectionException, UnexpectedException);
  /** Checks the database name */
  const std::wstring& checkDatabaseName(const std::wstring&) const
      throw (ConnectionException, UnexpectedException);
  /** Gives the database name */
  std::wstring        getDatabaseName() const { return database_name; }
  /** Gives the user name */
  std::wstring        getUserName() const { return user_name; }
  /** Gives the user password */
  std::wstring        getUserPass() const { return user_pass; }
  /** Gives the controller pool */
  AbstractControllerPool& getControllerPool();
  /** Inform ControllerPoolManager that we don't need controller_pool anymore */
  void                releaseControllerPool();
  /** Tell whether the connection must remain persistent */
  bool                getPersistentConnection() const { return persistent_connections; }
  /** Whether the controller should retrieve SQL warnings */
  bool                getRetrieveSQLWarnings() const { return retrieve_sql_warnings; }
private:
  /** List of controllers to connect to */
  std::vector<ControllerInfo> controller_list;
  /** Name of the database */
  std::wstring        database_name;
  /** User name for database access */
  std::wstring        user_name;
  /** User password for database access */
  std::wstring        user_pass;
  /** Controller connection policy */
  ConnectPolicy       connect_policy;
  /** Controller pool obtained from controller list and connect policy */
  AbstractControllerPool* controller_pool;
  /**
   * Whether a connection in autoCommit mode should remain persistent on cluster
   * backends
   */
  bool                persistent_connections;
  /** Whether to retrieve the sql warnings */
  bool                retrieve_sql_warnings;
};

} //namespace CarobNS
#endif /*CONNECTIONPARAMETERS_H_*/
