
/*
 * Sequoia: Database clustering technology Copyright 2002-2005
 * Continuent, Inc.

 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License. You
 * may *obtain a copy of the License at

 * http://www.apache.org/licenses/LICENSE-2.0 

 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.

 * Initial developer(s): Marc Herbert
 * Contributor(s):
 */

#ifndef CAROB_STRING_CODEC_HPP_
#define CAROB_STRING_CODEC_HPP_

#include "CarobException.hpp"

#include <locale>

namespace CarobNS {

/**
 * Exception thrown when an encoding error occurs.
 */
class CodecException : public DriverException
{
public:
  /**
   * Constructs a CodecException with the given message
   * @param s error message of the exception
   */
  CodecException(std::wstring s) : DriverException(s, L"22018") {};
  // In theory 22018 is only for SQL data conversions, and there is
  // a slight chance that we were doing something else. Even in this
  // case it is still more informative than nothing.
};


typedef std::codecvt<wchar_t, char, mbstate_t> mbs_codecvt;
typedef std::codecvt_byname<wchar_t, char, mbstate_t> mbs_codecvt_byname;

// This class should be splitted into interface and implementation,
// the "MBS" part of being restricted to the implementation.

// For a direct access to iconv read this:
// file://localhost/usr/share/doc/gcc-4.0-base/libstdc++/html/22_locale/codecvt.html

// This code probably does not work with UTF-16 wide chars, only UCS-2
// and UCS-4

std::locale trylocale(const char * const name) throw (CodecException);

/**
 * Multi-Byte String encoder and decoders according to a given or default locale
 */
class MBSCodec
{
  // TODO: replace loc by an accessor, so we could dynamically use
  // the (possibly changing) current locale() instead of a constant
  // one.

  // Here order matters (see init below)
  const std::locale loc; // hold our own (and cheap) copy for safety
  const mbs_codecvt& codecvt;

public:
  /** Encodes the given string using the locale set in the constructor */
  std::string encode(const std::wstring&) const throw (CodecException);
  /** Decodes the given string using the locale set in the constructor */
  std::wstring decode(const std::string&) const throw (CodecException);

  /**
   * Default constructor for the default converter, using
   * user-defined locale (typically LANG, LC_etc).  If you want the
   * C locale instead of this LANG-one use for instance this CTOR:
   *   MBSCodec(locale::classic())
   * or if you want the current (global) locale:
   *   MBSCodec(locale())
   */
  MBSCodec() :
      loc(std::locale("")),
      codecvt (std::use_facet<mbs_codecvt>(loc))
  { }

  /**
   * Use locarg locale for conversions
   * @param locarg locale to be used
   */
  MBSCodec(const std::locale& locarg) :
      loc(locarg),
      codecvt (std::use_facet<mbs_codecvt>(loc))
  { }

  /**
   * Use locale named locname for conversions
   * @param locname name of the locale to be used
   */
  MBSCodec(const char* locname) throw (CodecException) :
      loc(trylocale(locname)),
      codecvt (std::use_facet<mbs_codecvt>(loc))
  { }
  
};

/** 
 * The only purpose of this class is to prevent direct access to the
 * private codecs fields, while still be able to inline the calls.
 * Since the methods below rely on some static initialization, they
 * should NOT be used at initialization time!
 */
class StaticCodecs
{
public:
  /** Converts UTF-8 encoded string to a wide string */
  static std::wstring fromUTF8(const std::string& in) throw (CodecException)
  { return utf8_codec.decode(in); }
  /** Converts a wide string to a UTF-8 encoded string */
  static std::string toUTF8(const std::wstring& in) throw (CodecException)
  { return utf8_codec.encode(in); }
  /** Converts user's locale encoded string to a wide string */
  static std::wstring fromString(const std::string& in) throw (CodecException)
  { return user_codec.decode(in); }
  /** Converts a wide string to a user's locale encoded string */
  static std::string toString(const std::wstring& in) throw (CodecException)
  { return user_codec.encode(in); }
private:
  /** Codec for UTF-8 strings */
  static const CarobNS::MBSCodec utf8_codec;
  /** User-defined locale (typically set using LANG, LC_etc) codec */
  static const CarobNS::MBSCodec user_codec;
};

/**
 * Default (locale) String to Wide-String conversion. Since this method relies
 * on some static initialization, it should NOT be used at initialization
 * time!
 * @param in original string
 * @return converted wstring
 */
inline std::wstring fromString(const std::string& in) throw (CodecException)
{
  return StaticCodecs::fromString(in);
}
/**
 * Wide-String to default (locale) String conversion. Since this method relies
 * on some static initialization, it should NOT be used at initialization
 * time!
 * FIXME: throw exception on error
 * @param in original wide string
 * @return converted string
 */
inline std::string toString(const std::wstring& in) throw (CodecException)
{
  return StaticCodecs::toString(in);
}

inline std::wstring fromUTF8(const std::string& in) throw (CodecException)
{
  return StaticCodecs::fromUTF8(in);
}

inline std::string toUTF8(const std::wstring& in)  throw (CodecException)
{
  return StaticCodecs::toUTF8(in);
}

} // namespace CarobNS

#endif // include only once

/*
 * Local Variables:
 * c-file-style: "bsd"
 * c-basic-offset: 4
 * indent-tabs-mode: nil
 * End:
 */
