/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005 Emic Networks
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Gilles Rayrat
 * Contributor(s): Marc Herbert
 */

#include "Common.hpp"

#include "StringCodecs.hpp"

//LOG4CXX includes
#ifdef CAROB_USE_LOG4CXX
#include "StringCodecs.hpp" //for toString
#include <log4cxx/logger.h>
#endif

#ifndef __MINGW32__
#include <stdint.h> // for uint8_t, int32_t, uint32_t, int64_t, uint64_t
// Alternative:
// #include <cstdint>

// Warning: this header and all the types it defines (including "long
// long") are part of C99 and are MISSING from C++98.

// See:
// - "Incompatibilities Between ISO C and ISO C++" - David R. Tribble
// - C++ Technical Report 1 (TR1)
#endif
#include <iostream>
#include <sstream> //for wostringstream
#include <stdexcept>
#include <locale>

CarobNS::LogLevel CarobNS::currentLogLevel = LOG_LEVEL_WARN;

using std::wstring;
using std::string;
using std::wcerr;
using std::endl;

//LOG4CXX
#ifdef CAROB_USE_LOG4CXX
using namespace log4cxx;
namespace
{
  LoggerPtr logger(Logger::getLogger("Carob"));
}
#endif


// On linux or MACOSX>10.4 try "locale -a" to get the list of all available locales.
// TODO: it would be nice to be able to get this at runtime
// (in a non-portable way quite obviously...)

// An empty string "" will get the "default" C++ locale, typically set
// at run-time using environnement variables like LANG/LC_XXX. This
// will work fine as long as this "default" is set to some UTF8 locale.

// Using "C" should work fine as long as every string involved is pure
// ASCII.

#define NAME_OF_ANY_UTF8_LOCALE_AVAILABLE "en_US.utf8"

void CarobNS::setLogLevel(const LogLevel l)
{
#ifdef CAROB_USE_LOG4CXX
  if (l == LOG_LEVEL_DEBUG)
    logger->setLevel(log4cxx::Level::DEBUG);
  else if (l == LOG_LEVEL_INFO)
    logger->setLevel(log4cxx::Level::INFO);
  else if (l == LOG_LEVEL_WARN)
    logger->setLevel(log4cxx::Level::WARN);
  else if (l == LOG_LEVEL_ERROR)
    logger->setLevel(log4cxx::Level::ERROR);
  else if (l == LOG_LEVEL_FATAL)
    logger->setLevel(log4cxx::Level::FATAL);
  else if (l == LOG_LEVEL_OFF)
    logger->setLevel(log4cxx::Level::OFF);
#endif
  CarobNS::currentLogLevel = l;
}

void CarobNS::logDebug(const wstring& fctName, const wstring& s)
{
  if (isDebugEnabled())
  {
#ifdef CAROB_USE_LOG4CXX
    // Log4cxx <= 0.9.7 doesn't accept wstrings - we have to convert the string
    std::string tmp = CarobNS::StaticCodecs::toString(fctName + L": " + s);
    LOG4CXX_DEBUG(logger, tmp);
#else
    wcerr << DEBUG_PRE << L'(' << fctName << L") " << s << endl;
#endif
  }
}
void CarobNS::logInfo(const wstring& fctName, const wstring& s)
{
  if (isInfoEnabled())
  {
#ifdef CAROB_USE_LOG4CXX
    std::string tmp = CarobNS::StaticCodecs::toString(fctName + L": " + s);
    LOG4CXX_INFO(logger, tmp);
#else
    wcerr << INFO_PRE << L'(' << fctName << L") " << s << endl;
#endif
  }
}
void CarobNS::logWarn(const wstring& fctName, const wstring& s)
{
  if (isWarnEnabled())
  {
#ifdef CAROB_USE_LOG4CXX
    std::string tmp = CarobNS::StaticCodecs::toString(fctName + L": " + s);
    LOG4CXX_WARN(logger, tmp);
#else
    wcerr << WARN_PRE << L'(' << fctName << L") " << s << endl;
#endif
  }
}
void CarobNS::logError(const wstring& fctName, const wstring& s)
{
  if (isErrorEnabled())
  {
#ifdef CAROB_USE_LOG4CXX
    std::string tmp = CarobNS::StaticCodecs::toString(fctName + L": " + s);
    LOG4CXX_ERROR(logger, tmp);
#else
    wcerr << ERROR_PRE << L'(' << fctName << L") " << s << endl;
#endif
  }
}
void CarobNS::logFatal(const wstring& fctName, const wstring& s)
{
  if (isFatalEnabled())
  {
#ifdef CAROB_USE_LOG4CXX
    std::string tmp = CarobNS::StaticCodecs::toString(fctName + L": " + s);
    LOG4CXX_FATAL(logger, tmp);
#else
    wcerr << FATAL_PRE << L'(' << fctName << L") " << s << endl;
#endif
  }
}

bool CarobNS::isDebugEnabled()
{
  return CarobNS::currentLogLevel <= LOG_LEVEL_DEBUG;
}
bool CarobNS::isInfoEnabled()
{
  return CarobNS::currentLogLevel <= LOG_LEVEL_INFO;
}
bool CarobNS::isWarnEnabled()
{
  return CarobNS::currentLogLevel <= LOG_LEVEL_WARN;
}
bool CarobNS::isErrorEnabled()
{
  return CarobNS::currentLogLevel <= LOG_LEVEL_ERROR;
}
bool CarobNS::isFatalEnabled()
{
  return CarobNS::currentLogLevel <= LOG_LEVEL_FATAL;
}

wstring CarobNS::trim(const wstring& source, 
             const wchar_t* delims)
{
  wstring result(source);
  string::size_type index = result.find_last_not_of(delims);
  if(index != string::npos)
    result.erase(++index);

  index = result.find_first_not_of(delims);
  if(index != string::npos)
    result.erase(0, index);
  else
    result.erase();
  return result;
}

namespace {
  using namespace CarobNS;

  std::locale tryuserlocale();

  std::locale tryUTF8locale()
  {
    // __func__ is unfortunately C99
    wstring funcname(__WFILE__ L":tryUTF8locale()");

    // 1. trying hardwired macro
    try {
      logInfo(funcname,
              L"1. trying to use locale: " NAME_OF_ANY_UTF8_LOCALE_AVAILABLE);
      return trylocale(NAME_OF_ANY_UTF8_LOCALE_AVAILABLE);
    } catch (CodecException& ce) {
      logWarn(funcname,
              L"please install the " NAME_OF_ANY_UTF8_LOCALE_AVAILABLE " locale");
    }

    // 2. try to fallback on the user defined locale
    string user_loc;
    try{
      user_loc = std::locale("").name();
    } catch (std::runtime_error&) {
      user_loc = std::locale().name();
    }
    logInfo(funcname,
            std::wstring(L"2. trying to fallback on locale: ")
            + fromString(user_loc));

    try { // search for "utf8" or variants

      string::size_type utf8pos;

      // sorry for this
      (utf8pos = user_loc.rfind("UTF")) != string::npos 
        || (utf8pos = user_loc.rfind("Utf")) != string::npos 
        || ( utf8pos = user_loc.rfind("utf")) != string::npos;

      if (utf8pos != string::npos)
        if (user_loc.at(utf8pos+3) == '8'
            || user_loc.at(utf8pos+4) == '8')
          return std::locale(user_loc.c_str());

    } catch (std::out_of_range& oor) {
      // out_of_range => not found
    }

    logWarn(funcname, fromString(user_loc)
            + L" does not seem to be an utf8 locale either");


    // 3. Else try to hack the user locale name a bit
    string::size_type delimpos;
    string nocodeset;
    if ((delimpos = user_loc.find_last_of('.')) != string::npos)
      nocodeset = user_loc.substr(0, delimpos);
    else if ((delimpos = user_loc.find_last_of('@')) != string::npos)
      nocodeset = user_loc.substr(0, delimpos);
    else
      nocodeset = user_loc;

    string hacked_names[2];
    hacked_names[0] = nocodeset + ".UTF-8";
    hacked_names[1] = nocodeset + ".utf8";

    for (int i=0; i<2; i++)
      try {
        logInfo(funcname, std::wstring(L"3n. trying to fall back on locale ")
                + fromString(hacked_names[i]));
        return trylocale(hacked_names[i].c_str());
      } catch (CodecException&) {
        logWarn(funcname,
                std::wstring(L"or please install locale ")
                + fromString(hacked_names[i]));
      }

    // 4. nothing found
    logFatal(funcname,
             std::wstring(L"Please install one of the following locales: "
                          NAME_OF_ANY_UTF8_LOCALE_AVAILABLE ", ")
             + fromString(user_loc) + L", "
             + fromString(nocodeset)
             + L".utf8"
      );
    throw CodecException(L"no UTF8 codec found");

    // TODO: fall back on:
    // - configuration file
    // - iconv
    // - MultiByteToWideChar(CP_UTF8,...)
    // - others?
    // See CAROB-74

  } // tryUTF8locale()

  std::locale tryuserlocale()
  {
    try {
      return std::locale("");
    } catch (std::runtime_error& ) {
      logWarn(__WFILE__ L":tryuserlocale()",
              L"Missing user-preferred locale (check LANG). "
              "Falling back on current global locale.");
      return std::locale();
    }
  }
}

// init order matters here, since a failure to init utf8_codec will
// call user_codec
const CarobNS::MBSCodec CarobNS::StaticCodecs::user_codec(tryuserlocale());
const CarobNS::MBSCodec CarobNS::StaticCodecs::utf8_codec(tryUTF8locale());

// decimal point initialization
wchar_t CarobNS::userDecimalPoint =
    std::use_facet<std::numpunct<wchar_t> >(tryuserlocale()).decimal_point();
wchar_t CarobNS::controllerDecimalPoint = 
    std::use_facet<std::numpunct<wchar_t> >(std::locale::classic()).decimal_point();

//Can this be factorized with ParameterStatement ?
template <class T> wstring CarobNS::toWString(const T& t)
{
  std::wostringstream buffer;
  buffer.imbue(std::locale::classic());
  buffer << t;
  return buffer.str();        
}
// Explicit instantiation for types actually used
// Don't forget to update javadoc when changing this
template wstring CarobNS::toWString<char>(const char&);
template wstring CarobNS::toWString<wchar_t>(const wchar_t&);
template wstring CarobNS::toWString<int16_t>(const int16_t&);
template wstring CarobNS::toWString<uint16_t>(const uint16_t&);
template wstring CarobNS::toWString<int32_t>(const int32_t&);
template wstring CarobNS::toWString<uint32_t>(const uint32_t&);
template wstring CarobNS::toWString<int64_t>(const int64_t&);
template wstring CarobNS::toWString<uint64_t>(const uint64_t&);
template wstring CarobNS::toWString<float>(const float&);
template wstring CarobNS::toWString<double>(const double&);

// locale dependant conversions
template <class T> wstring CarobNS::toUserString(const T& t)
{
  std::wostringstream buffer;
  // imbue with user's locale
  buffer.imbue(std::locale());
  buffer << t;
  return buffer.str();        
}

template wstring CarobNS::toUserString<char>(const char&);
template wstring CarobNS::toUserString<wchar_t>(const wchar_t&);
template wstring CarobNS::toUserString<int16_t>(const int16_t&);
template wstring CarobNS::toUserString<uint16_t>(const uint16_t&);
template wstring CarobNS::toUserString<int32_t>(const int32_t&);
template wstring CarobNS::toUserString<uint32_t>(const uint32_t&);
template wstring CarobNS::toUserString<int64_t>(const int64_t&);
template wstring CarobNS::toUserString<uint64_t>(const uint64_t&);
template wstring CarobNS::toUserString<float>(const float&);
template wstring CarobNS::toUserString<double>(const double&);

template <class T> bool CarobNS::wstringTo(T& t, const std::wstring& s)
{
  std::wistringstream iss(s);
  return !(iss >> std::skipws >> t).fail();
}
// Explicit instantiation for types actually used
// Don't forget to update javadoc when changing this
template bool CarobNS::wstringTo<int>(int&, const std::wstring&);
template bool CarobNS::wstringTo<int64_t>(int64_t&, const std::wstring&);
template bool CarobNS::wstringTo<uint64_t>(uint64_t&, const std::wstring&);
template bool CarobNS::wstringTo<float>(float&, const std::wstring&);
template bool CarobNS::wstringTo<double>(double&, const std::wstring&);

wstring CarobNS::replaceAll(wstring src, const wstring &from, const wstring &to)
{
  wstring::size_type found = src.find(from);
  while (found != wstring::npos)
  {
    src.replace(found, from.length(), to);
    found = src.find(from, found+to.length());
  }
  return src;
}

/*
 * Local Variables:
 * c-file-style: "bsd"
 * c-basic-offset: 2
 * indent-tabs-mode: nil
 * End:
 */
