/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005-2006 Continuent, Inc.
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Gilles Rayrat
 * Contributor(s): Marc Herbert, Zsolt Simon
#ifdef CAROB_USE_POLL
 * Contributors += Bruno Corsi dos Santos
#endif
 */

#include "ControllerPinger.hpp"
#include "ConnectionParameters.hpp"
#include "JavaSocket.hpp"
#include "Common.hpp"
#include "Connection.hpp"

#ifdef CAROB_USE_POLL
#include <sys/poll.h>
#define POLL_FUNCTION L"Poll"
#else
#define POLL_FUNCTION L"Select"
#endif

#include <vector>

// library-wide objects
namespace
{
  /**
   * List of controllers suspected of failure.<br>
   * This list is library-wide, shared by all clients (with a synchronization
   * on <code>suspected_controllers_CS</code>)
   */
  std::vector<CarobNS::ControllerInfo> suspected_controllers;
  /** mutex on suspected controllers */
  CarobNS::CriticalSection             suspected_controllers_CS;
  /** # of controllers that have fail since this process has been created */
  int                                   controller_failure_number = 0;
};

using namespace CarobNS;
using std::wstring;
/**
 * Utility struct to associate a controller to its ping socket. Used by #run()
 */
typedef struct
{
  ControllerInfo  controllerInfo;
  JavaSocket*     pingSocketPtr;
} SuspectController;

// init instance only once
ControllerPinger* ControllerPinger::unique_instance_ptr = NULL;

/* static */
ControllerPinger& ControllerPinger::getInstance()
{
  if (unique_instance_ptr == NULL)
    unique_instance_ptr = new ControllerPinger();
  return *unique_instance_ptr;
}

void ControllerPinger::suspectControllerOfFailure(
    ControllerInfo& controllerInfoPrm)
{
  wstring fctName(L"ControllerPinger::suspectControllerOfFailure");

  // First check that the controller is not already in the list of suspects
  if (isSuspectedOfFailure(controllerInfoPrm))
    return;

  LockScope scLs(&suspected_controllers_CS);
  //just for reporting (in the future ?)
  controller_failure_number++;
  // Add the controller (a copy of the given controller info) to the list
  suspected_controllers.push_back(controllerInfoPrm);
  if (isDebugEnabled())
    logDebug(fctName, L"Controller " + static_cast<wstring>(controllerInfoPrm)
        + L" is now suspected of failure (list size="
        + toUserString(suspected_controllers.size())+L")");
}

bool ControllerPinger::isSuspectedOfFailure(const ControllerInfo& controllerInfoPrm)
{
  // This is a good place to do the ping on suspected controllers, we are
  // called by most of the functions in this class
  run();

  LockScope scLs(&suspected_controllers_CS);
  // Just compare controller info with each item in the whole list
  for (std::vector<ControllerInfo>::const_iterator iter = suspected_controllers.begin();
      iter != suspected_controllers.end(); iter++)
  {
    if (*iter == controllerInfoPrm)
      return true;
  }
  return false;
}

void ControllerPinger::removeControllerFromSuspectList(
    const ControllerInfo& controllerInfoPrm)
{
  wstring fctName(L"ControllerPinger::removeControllerFromSuspectList");
  LockScope scLs(&suspected_controllers_CS);

  for (std::vector<ControllerInfo>::iterator iter = suspected_controllers.begin();
      iter != suspected_controllers.end(); iter++)
  {
    if (*iter == controllerInfoPrm)
    {
      suspected_controllers.erase(iter);
      if (isDebugEnabled())
      {
        logDebug(fctName, L"Controller " + static_cast<wstring>(controllerInfoPrm)
            + L" has been removed from suspect list (new list size="
            + toUserString(suspected_controllers.size())+L")");
      }
      return;
    }
  }
  if (isWarnEnabled())
  {
    logWarn(fctName, L"Controller " + static_cast<wstring>(controllerInfoPrm)
        + L" is not (anymore?) in the suspect list");
  }
}

void ControllerPinger::run()
{
  LockScope scLs(&suspected_controllers_CS);

  wstring fctName(L"ControllerPinger::run");
  if (suspected_controllers.size() < 1)
    return;

  // list of suspected controllers we can connect to 
  std::vector<SuspectController> suspected_and_connected_controllers;
#ifdef CAROB_USE_POLL
  //create the structures to use poll syscall
  unsigned int nfds = 0;
  unsigned int maxnfds = suspected_controllers.size();
  // Allocate the largest number that nfds can reach
  struct pollfd *ufds = static_cast<struct pollfd *> (calloc(maxnfds, sizeof(struct pollfd)));
  if(!ufds)
  {
    // Couldn't alloc the memory...
    if (isErrorEnabled())
    {
      logError(fctName, L"Calloc of poll fds failed");
    }
    return;
  }
#else
  //create set of fds for polling
  fd_set writableControllers;
  FD_ZERO(&writableControllers);
  struct timeval tv;
  tv.tv_sec = 0;
  tv.tv_usec = poll_timeout_ms*1000; // milli -> micro seconds conversion
  // fdMax is *not* the number of fds, but the maximum value of created sockets
  int fdMax = -1;
#endif
  // return of polling function
  int retVal = 0;
  // Ping each controller and add the ping socket in the set of fds for polling
  for (std::vector<ControllerInfo>::iterator iter = suspected_controllers.begin();
      iter != suspected_controllers.end(); iter++)
  {
    SuspectController sc;
    sc.pingSocketPtr = NULL; // prevents bad deletes 
    try
    {
      if (isDebugEnabled())
        logDebug(fctName, L"Creating ping socket to suspected controller...");
      // creates and prepares a new socket for pinging the controller
      sc.pingSocketPtr = new JavaSocket();
      sc.pingSocketPtr->create(false); //create non blocking
      sc.controllerInfo = *iter;
      //connect socket and add it to the list of fds
      if (sc.pingSocketPtr->connectTo(iter->getHostName(), iter->getHostPort()))
      {
        // no exception => add the controller to the suspects we are connected to
        suspected_and_connected_controllers.push_back(sc);
        // add its socket to the polling set
        int sock = sc.pingSocketPtr->getFd();
#ifdef CAROB_USE_POLL
        ufds[nfds].fd = sock;
        ufds[nfds].events = POLLOUT;
        nfds++;
        // The ufds[nfds].revents is is going to be modified
#else
        if (sock > fdMax)
          fdMax = sock;
        FD_SET(sock, &writableControllers);
#endif
      }
    }
    catch (...)
    {
      delete sc.pingSocketPtr;
    }
  }
#ifdef CAROB_USE_POLL
  retVal = poll(ufds, nfds, poll_timeout_ms);
#else
  retVal = select(fdMax+1, NULL, &writableControllers, NULL, &tv);
#endif
  if (retVal == -1)
  {
    if (isErrorEnabled())
    {
      logError(fctName, wstring(POLL_FUNCTION) + L" returned error #"+toUserString(errno));
    }
    // clean-up will be done a the end of this function
  }
  // if (retVal == 0) => timeout, no controller up again, do nothing
  // If controllers are found up again, let's remove them from the list of
  // suspects
  if (retVal > 0)
  {
    // iterates through controller list to find the ones that are up again
#ifdef CAROB_USE_POLL
    int i=0;
#endif
    for (std::vector<SuspectController>::iterator iter = suspected_and_connected_controllers.begin();
        iter != suspected_and_connected_controllers.end(); iter++)
    {
#ifdef CAROB_USE_POLL
      if (ufds[i].revents & POLLOUT)
#else
      if (FD_ISSET(iter->pingSocketPtr->getFd(), &writableControllers))
#endif
      {
        // send ping command, upon wich controller closes its connection with
        // the client that issued this ping
        iter->pingSocketPtr->writeJavaInt(Ping);
        removeControllerFromSuspectList(iter->controllerInfo);
      }
#ifdef CAROB_USE_POLL
      i++;
#endif
    }
  }

  // clean-up
  for (std::vector<SuspectController>::iterator iter = suspected_and_connected_controllers.begin();
    iter != suspected_and_connected_controllers.end(); iter++)
  {
    delete iter->pingSocketPtr;
  }
  suspected_and_connected_controllers.clear();
#ifdef CAROB_USE_POLL
  free(ufds);
#endif
}

/*static*/
int ControllerPinger::getNumberOfFailures()
{
  return controller_failure_number;
}
