/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005-2006 Continuent, Inc.
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Gilles Rayrat
 * Contributor(s): 
 */

#include "TestControllerPool.hpp"

// not to derive: just to get default parameters
#include "../ConnectionSetup.hpp"

#include "ControllerPool.hpp"
#include "ControllerPinger.hpp"
#include "Statement.hpp"
#include "Connection.hpp"

#include "Common.hpp"

#include <string>
#include <iostream>

using std::wstring;

using namespace CarobNS;

void TestControllerPool::setUp()
{
}

void TestControllerPool::tearDown()
{
}

void TestControllerPool::testRoundRobinSequence()
{
  wstring fctName(L"TestControllerPool::testRoundRobinSequence");

  ControllerInfo c1(ConnectionSetup::DEFAULT_HOST, 4000);
  ControllerInfo c2(ConnectionSetup::DEFAULT_HOST, 4001);
  ControllerInfo c3(ConnectionSetup::DEFAULT_HOST, 4002);
  ControllerInfo c4(ConnectionSetup::DEFAULT_HOST, 4003);
  
  std::vector<ControllerInfo> v;
  v.push_back(c1);
  v.push_back(c2);
  v.push_back(c3);
  v.push_back(c4);

  RoundRobinControllerPool cp(v);
  CPPUNIT_ASSERT(cp.getController() == c1);
  ControllerPinger::getInstance().suspectControllerOfFailure(c2);
  ControllerPinger::getInstance().suspectControllerOfFailure(c4);
  CPPUNIT_ASSERT(cp.getController() == c3);
  CPPUNIT_ASSERT(cp.getController() == c1);
}

void TestControllerPool::testNoMoreController()
{
  wstring fctName(L"TestControllerPool::testNoMoreController");

  ControllerInfo c1(ConnectionSetup::DEFAULT_HOST, 4000);
  ControllerInfo c2(ConnectionSetup::DEFAULT_HOST, 4001);
  ControllerInfo c3(ConnectionSetup::DEFAULT_HOST, 4002);
  ControllerInfo c4(ConnectionSetup::DEFAULT_HOST, 4003);

  std::vector<ControllerInfo> v;
  v.push_back(c1);
  v.push_back(c2);
  v.push_back(c3);
  v.push_back(c4);

  RoundRobinControllerPool cp(v);
  ControllerPinger::getInstance().suspectControllerOfFailure(c1);
  ControllerPinger::getInstance().suspectControllerOfFailure(c2);
  ControllerPinger::getInstance().suspectControllerOfFailure(c3);
  ControllerPinger::getInstance().suspectControllerOfFailure(c4);
  
  try
  {
    if (isInfoEnabled())
      logInfo(fctName, L"Trying to get controller - should fail");
    cp.getController();
    CPPUNIT_ASSERT(false); //should have thrown an exception
  }
  catch (NoMoreControllerException nmce)
  {
    if (isErrorEnabled())
      logError(fctName, L"getController() failed (this is ok). Exception: "
          + nmce.description());
  }
}


void TestControllerPool::testSuspectListUpdate()
{
  wstring fctName(L"TestControllerPool::testSuspectListUpdate");
  std::wcerr<<L"Launch 4 servers on localhost:4000-4003 and press <return>"<<std::endl;
  std::cin.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
  
  ControllerInfo c1(ConnectionSetup::DEFAULT_HOST, 4000);
  ControllerInfo c2(ConnectionSetup::DEFAULT_HOST, 4001);
  ControllerInfo c3(ConnectionSetup::DEFAULT_HOST, 4002);
  ControllerInfo c4(ConnectionSetup::DEFAULT_HOST, 4003);
  
  std::vector<ControllerInfo> v;
  v.push_back(c1);
  v.push_back(c2);
  v.push_back(c3);
  v.push_back(c4);

  RoundRobinControllerPool cp(v);
  ControllerPinger::getInstance().suspectControllerOfFailure(c1);
  ControllerPinger::getInstance().suspectControllerOfFailure(c2);
  CPPUNIT_ASSERT(cp.getController() == c3);
  CPPUNIT_ASSERT(cp.getController() == c4);
  ControllerPinger::getInstance().run();
  CPPUNIT_ASSERT(cp.getController() == c1);
  CPPUNIT_ASSERT(cp.getController() == c2);
}

CppUnit::Test* TestControllerPool::suite()
{
  CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "TestControllerPool" );
  suiteOfTests->addTest(new CppUnit::TestCaller<TestControllerPool>(
                                 "TestControllerPool::testRoundRobinSequence",
                                 &TestControllerPool::testRoundRobinSequence));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestControllerPool>(
                                 "TestControllerPool::testNoMoreController",
                                 &TestControllerPool::testNoMoreController));
//Test disabled, needs user interaction
/*  suiteOfTests->addTest(new CppUnit::TestCaller<TestControllerPool>(
                                 "TestControllerPool::testSuspectListUpdate",
                                 &TestControllerPool::testSuspectListUpdate));
*/
  return suiteOfTests;
}
