/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005-2006 Continuent, Inc.
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Gilles Rayrat
 * Contributor(s): 
 */

#include "TestExecWriteRequest.hpp"

#include "Statement.hpp"

#include "CarobException.hpp"
#include "Common.hpp"

#include <string>

using std::wstring;

using namespace CarobNS;

void TestExecWriteRequest::testWriteBadRequest()
{
  wstring fctName(L"TestExecWriteRequest::testWriteBadRequest");
  try
  {
    Statement* statementPtr = connectionPtr->createStatement();
    if (isInfoEnabled())
    {
      logInfo(fctName, L"Executing dummy update - should fail");
    }
    statementPtr->executeUpdate(L"dummy request");
    // We should receive an exception instead of coming here
    CPPUNIT_ASSERT(false);
  }
  catch (ControllerException ce)
  {
    if (isErrorEnabled())
    {
      logError(fctName, L"Update failed (this is ok). Exception: "
          + ce.description());
    }
  }
}

void TestExecWriteRequest::testWriteBadTable()
{
  wstring fctName(L"TestExecWriteRequest::testWriteBadTable");
  try
  {
    Statement* statementPtr = connectionPtr->createStatement();
    if (isInfoEnabled())
    {
      logInfo(fctName, L"Executing bad table update - this should fail");
    }
    statementPtr->executeUpdate(L"update dummy set name='gotit' where id=0");
    // We should receive an exception instead of coming here
    CPPUNIT_ASSERT(false);
  }
  catch (BackendException be)
  {
    if (isErrorEnabled())
    {
      logError(fctName, L"Update failed (this is ok). Exception: "
          + be.description());
    }
  }
}

void TestExecWriteRequest::testWriteGood()
{
  wstring fctName(L"TestExecWriteRequest::testWriteGood");
  Statement* statementPtr = connectionPtr->createStatement();
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Executing update - should succeed");
  }
  int nbRowsAffected = statementPtr->executeUpdate(
      L"update product set name='changed by testExecuteUpdateGood' where id=0");
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Update succeeded. Number of affected rows = "
        + toUserString(nbRowsAffected));
  }
  CPPUNIT_ASSERT(nbRowsAffected == 1);
}

void TestExecWriteRequest::testWriteThousandGood()
{
  wstring fctName(L"TestExecWriteRequest::testWriteThousandGood");
  wstring requestBegin = L"UPDATE product SET cost=";
  wstring requestEnd = L" WHERE id=";

  Statement* statementPtr = connectionPtr->createStatement();
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Executing 1000 writes");
  }
  for (int i=0; i<1000; i++)
  {
    int nbRowsAffected = statementPtr->executeUpdate(requestBegin + toWString(i)
                      + requestEnd + toWString(i%50));
    CPPUNIT_ASSERT(nbRowsAffected == 1);
  }
  if (isInfoEnabled())
  {
    logInfo(fctName, L"1000 writes succeeded");
  }
}

CppUnit::Test* TestExecWriteRequest::suite()
{
  CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "TestExecWriteRequest" );

  suiteOfTests->addTest(new CppUnit::TestCaller<TestExecWriteRequest>(
                                 "TestExecWriteRequest::testWriteBadRequest", 
                                 &TestExecWriteRequest::testWriteBadRequest));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestExecWriteRequest>(
                                 "TestExecWriteRequest::testWriteBadTable", 
                                 &TestExecWriteRequest::testWriteBadTable));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestExecWriteRequest>(
                                 "TestExecWriteRequest::testWriteGood", 
                                 &TestExecWriteRequest::testWriteGood));
/*  suiteOfTests->addTest(new CppUnit::TestCaller<TestExecWriteRequest>(
                                 "TestExecWriteRequest::testWriteThousandGood", 
                                 &TestExecWriteRequest::testWriteThousandGood));
*/
  return suiteOfTests;
}
