/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005-2006 Continuent, Inc.
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Gilles Rayrat
 * Contributor(s): 
 */

#include "TestExec.hpp"

#include "ResultSetMetaData.hpp"
#include "Statement.hpp"

#include "CarobException.hpp"
#include "Common.hpp"

#include <string>
#include <iostream>

using std::wstring;
using std::endl;

using namespace CarobNS;

void TestExec::testExecuteBadRequests()
{
  wstring fctName(L"TestStatement::testExecuteBadRequests");
  Statement* statementPtr = NULL;
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Testing execute with bad query \"dummy\" (should fail)");
  }
  try
  {
    statementPtr = connectionPtr->createStatement();
    statementPtr->execute(L"dummy");
    // We should receive an exception instead of coming here
    CPPUNIT_ASSERT(false);
  }
  catch (BackendException be)
  {
    if (isErrorEnabled())
    {
      logError(fctName, L"Read failed (this is ok). Exception: "
          + be.description());
    }
  }
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Testing execute with bad query \"SELECT * FROM dummy\" (should fail)");
  }
  try
  {
    statementPtr->execute(L"SELECT * FROM dummy");
    // We should receive an exception instead of coming here
    CPPUNIT_ASSERT(false);
  }
  catch (BackendException be)
  {
    if (isErrorEnabled())
    {
      logError(fctName, L"Read failed (this is ok). Exception: "
          + be.description());
    }
  }
}

void TestExec::testExecuteWithSelect()
{
  wstring fctName(L"TestExec::testExecuteWithSelect");
  Statement* statementPtr = NULL;
  statementPtr = connectionPtr->createStatement();
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Executing execute(SELECT * FROM address) - should succeed");
  }
  bool isRS = statementPtr->execute(L"SELECT * FROM address");
  CPPUNIT_ASSERT(isRS);
  
  DriverResultSet* drsPtr = statementPtr->getResultSet();
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Read succeeded. Displaying 50 rows:");
  }
  //Display 50 rows using metadata and toString
  ResultSetMetaData rsmd(drsPtr);
  std::wostringstream buffer;
  for (int i=0; i<50; i++)
  {
    drsPtr->next();
    buffer<<i+1<<L"\t";
    for (int j=0; j<rsmd.getColumnCount(); j++)
      buffer<<drsPtr->getAsString(j+1)<<L"\t";
    buffer<<endl;
  }
  if (isInfoEnabled())
  {
    logInfo(fctName, buffer.str());
  }
}
void TestExec::testExecuteWithUpdate()
{
  wstring fctName(L"TestExec::testExecuteWithUpdate");
  Statement* statementPtr = NULL;
  statementPtr = connectionPtr->createStatement();
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Executing execute(UPDATE FROM address...) - should succeed");
  }
  bool isRS = statementPtr->execute(L"UPDATE product SET name='changed by testExecuteWithUpdate' WHERE id=0");
  CPPUNIT_ASSERT(!isRS);
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Update with execute succeeded. Number of affected rows = "
        + toUserString(statementPtr->getUpdateCount()));
  }
  CPPUNIT_ASSERT(statementPtr->getUpdateCount() == 1);
}

void TestExec::testExecuteWithSelectStreamed()
{
  wstring fctName(L"TestExec::testExecuteWithSelectStreamed");
  Statement* statementPtr = NULL;
  statementPtr = connectionPtr->createStatement();
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Executing streamed execute(SELECT * FROM address) - should succeed");
  }
  statementPtr->setFetchSize(1);
  bool isRS = statementPtr->execute(L"SELECT * FROM address");
  CPPUNIT_ASSERT(isRS);
  
  DriverResultSet* drsPtr = statementPtr->getResultSet();
  ResultSetMetaData rsmd(drsPtr);
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Read succeeded. Displaying 50 rows:");
  }
  //Display rows using metadata and toString
  std::wostringstream buffer;
  while (drsPtr->next())
  {
    ResultSetMetaData rsmd(drsPtr);
    for (int j=0; j<rsmd.getColumnCount(); j++)
      buffer<<drsPtr->getAsString(j+1)<<L"\t";
    buffer<<endl;
  }
  if (isInfoEnabled())
  {
    logInfo(fctName, buffer.str());
  }
}

CppUnit::Test* TestExec::suite()
{
  CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "TestExec" );
  suiteOfTests->addTest(new CppUnit::TestCaller<TestExec>(
                                 "TestExec::testExecuteBadRequests", 
                                 &TestExec::testExecuteBadRequests));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestExec>(
                                 "TestExec::testExecuteWithSelect", 
                                 &TestExec::testExecuteWithSelect));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestExec>(
                                 "TestExec::testExecuteWithUpdate", 
                                 &TestExec::testExecuteWithUpdate));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestExec>(
                                 "TestExec::testExecuteWithSelectStreamed", 
                                 &TestExec::testExecuteWithSelectStreamed));

  return suiteOfTests;
}
