/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005-2006 Continuent, Inc.
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Gilles Rayrat
 * Contributor(s): 
 */

#include <iostream>

#include "TestPreparedStatement.hpp"
#include "ParameterStatement.hpp"
#include "ResultSetMetaData.hpp"
#include "TypeTag.hpp"

using std::wstring;
using std::endl;

using namespace CarobNS;

void TestPreparedStatement::testPrepareEmptyStatement()
{
  wstring fctName(L"testPrepareEmptyStatement");
  Statement* statementPtr = NULL;
  try
  {
    statementPtr = connectionPtr->createStatement();
    if (isInfoEnabled())
    {
      logInfo(fctName, L"Executing getMetaData() on empty statement - should fail");
    }
    ResultSetMetaData* rsmdPtr = statementPtr->getMetaData();
    CPPUNIT_ASSERT(rsmdPtr == NULL); //just to avoid warning on unsused var
  }
  catch (BackendException be)
  {
    if (isErrorEnabled())
    {
      logError(fctName, L"Query failed (this is ok). Exception: "
          + be.description());
    }
  }
}
void TestPreparedStatement::testPrepareEmptyParameterStatement()
{
  wstring fctName(L"testPrepareEmptyParameterStatement");
  ParameterStatement* psPtr = NULL;
  try
  {
    psPtr = connectionPtr->createParameterStatement(L"");
    if (isInfoEnabled())
    {
      logInfo(fctName, L"Executing getMetaData() on empty parameter statement - should fail");
    }
    ResultSetMetaData* rsmdPtr = psPtr->getMetaData();
    CPPUNIT_ASSERT(rsmdPtr == NULL);
  }
  catch (BackendException be)
  {
    if (isErrorEnabled())
    {
      logError(fctName, L"Query failed (this is ok). Exception: "
          + be.description());
    }
  }
}

void TestPreparedStatement::testPrepareUpdateStatement()
{
  wstring fctName(L"testPrepareUpdateStatement");
  Statement* statementPtr = NULL;
  try
  {
    statementPtr = connectionPtr->createStatement();
    int nbRowsAffected = statementPtr->executeUpdate(
        L"update product set name='changed by testPrepareUpdate' where id=0");
    if (isInfoEnabled())
    {
      logInfo(fctName, L"Executing getMetaData() on update statement - should fail");
    }
    ResultSetMetaData* rsmdPtr = statementPtr->getMetaData();
    CPPUNIT_ASSERT(nbRowsAffected == 0 && rsmdPtr == NULL); //just to avoid warning on unsused var
  }
  catch (BackendException be)
  {
    if (isErrorEnabled())
    {
      logError(fctName, L"Query failed (this is ok). Exception: "
          + be.description());
    }
  }
}

void TestPreparedStatement::testPrepareUpdateParameterStatement()
{
  wstring fctName(L"testPrepareUpdateParameterStatement");
  ParameterStatement* psPtr = connectionPtr->createParameterStatement(L"update address set name='changed by testPrepareUpdateParameterStatement' where id=?");
  psPtr->setInt(1,0);
  psPtr->executeUpdate();
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Executing getMetaData() on update parameter statement - should return null");
  }
  ResultSetMetaData* rsmdPtr = psPtr->getMetaData();
  CPPUNIT_ASSERT(rsmdPtr == NULL);
}

void TestPreparedStatement::testPrepareExecuteQueryStatement()
{
  wstring fctName(L"testPrepareExecuteQueryStatement");
  Statement* statementPtr = connectionPtr->createStatement();
  statementPtr->executeQuery(L"SELECT * FROM address");
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Executing getMetaData() on query statement");
  }
  ResultSetMetaData* rsmdPtr = statementPtr->getMetaData();
  CPPUNIT_ASSERT(rsmdPtr != NULL); //just to avoid warning on unsused var
  CPPUNIT_ASSERT(rsmdPtr->getColumnCount() == 5);
}

void TestPreparedStatement::testPrepareExecuteQueryParameterStatement()
{
  wstring fctName(L"testPrepareExecuteQueryParameterStatement");
  ParameterStatement* psPtr = connectionPtr->createParameterStatement(L"SELECT * FROM address where id=?");
  psPtr->setInt(1,0);
  psPtr->executeQuery();
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Executing getMetaData() on query parameter statement");
  }
  ResultSetMetaData* rsmdPtr = psPtr->getMetaData();
  CPPUNIT_ASSERT(rsmdPtr != NULL);
  CPPUNIT_ASSERT(rsmdPtr->getColumnName(1) == L"ID");
}

void TestPreparedStatement::testPrepareExecuteStatement()
{
  wstring fctName(L"testPrepareExecuteStatement");
  Statement* statementPtr = connectionPtr->createStatement();
  statementPtr->execute(L"SELECT * FROM address");
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Executing getMetaData() on execute statement");
  }
  ResultSetMetaData* rsmdPtr = statementPtr->getMetaData();
  CPPUNIT_ASSERT(rsmdPtr != NULL); //just to avoid warning on unsused var
  CPPUNIT_ASSERT(rsmdPtr->getTableName(2) == L"ADDRESS");
}

void TestPreparedStatement::testPrepareExecuteParameterStatement()
{
  wstring fctName(L"testPrepareExecuteParameterStatement");
  ParameterStatement* psPtr = connectionPtr->createParameterStatement(L"SELECT * FROM address where id=?");
  psPtr->setInt(1,0);
  psPtr->execute();
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Executing getMetaData() on execute parameter statement");
  }
  ResultSetMetaData* rsmdPtr = psPtr->getMetaData();
  CPPUNIT_ASSERT(rsmdPtr != NULL);
  CPPUNIT_ASSERT(rsmdPtr->getColumnType(3) == SQLT_VARCHAR);
}

void TestPreparedStatement::testPrepareStatement()
{
  wstring fctName(L"testPrepareStatement");
  Statement* statementPtr = connectionPtr->createStatement();
  statementPtr->setRequest(L"SELECT * FROM address");
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Executing getMetaData() on statement");
  }
  ResultSetMetaData* rsmdPtr = statementPtr->getMetaData();
  CPPUNIT_ASSERT(rsmdPtr != NULL); //just to avoid warning on unsused var
  CPPUNIT_ASSERT(rsmdPtr->getTableName(2) == L"ADDRESS");
}

void TestPreparedStatement::testPrepareParameterStatement()
{
  wstring fctName(L"testPrepareParameterStatement");
  ParameterStatement* psPtr = connectionPtr->createParameterStatement(L"SELECT * FROM address where id=?");
  psPtr->setInt(1,0);
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Executing getMetaData() on parameter statement");
  }
  ResultSetMetaData* rsmdPtr = psPtr->getMetaData();
  CPPUNIT_ASSERT(rsmdPtr != NULL);
  CPPUNIT_ASSERT(rsmdPtr->getColumnType(3) == SQLT_VARCHAR);
}

CppUnit::Test* TestPreparedStatement::suite()
{
  CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "TestPreparedStatement" );
  suiteOfTests->addTest(new CppUnit::TestCaller<TestPreparedStatement>(
                                 "TestPreparedStatement::testPrepareEmptyStatement",
                                 &TestPreparedStatement::testPrepareEmptyStatement));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestPreparedStatement>(
                                 "TestPreparedStatement::testPrepareEmptyParameterStatement",
                                 &TestPreparedStatement::testPrepareEmptyParameterStatement));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestPreparedStatement>(
                                 "TestPreparedStatement::testPrepareUpdateStatement",
                                 &TestPreparedStatement::testPrepareUpdateStatement));
//TODO: re-enable this test once SEQUOIA-262 is fixed
/*  suiteOfTests->addTest(new CppUnit::TestCaller<TestPreparedStatement>(
                                 "TestPreparedStatement::testPrepareUpdateParameterStatement",
                                 &TestPreparedStatement::testPrepareUpdateParameterStatement));
*/
  suiteOfTests->addTest(new CppUnit::TestCaller<TestPreparedStatement>(
                                 "TestPreparedStatement::testPrepareExecuteQueryStatement",
                                 &TestPreparedStatement::testPrepareExecuteQueryStatement));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestPreparedStatement>(
                                 "TestPreparedStatement::testPrepareExecuteQueryParameterStatement",
                                 &TestPreparedStatement::testPrepareExecuteQueryParameterStatement));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestPreparedStatement>(
                                 "TestPreparedStatement::testPrepareExecuteStatement",
                                 &TestPreparedStatement::testPrepareExecuteStatement));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestPreparedStatement>(
                                 "TestPreparedStatement::testPrepareExecuteParameterStatement",
                                 &TestPreparedStatement::testPrepareExecuteParameterStatement));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestPreparedStatement>(
                                 "TestPreparedStatement::testPrepareStatement",
                                 &TestPreparedStatement::testPrepareStatement));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestPreparedStatement>(
                                 "TestPreparedStatement::testPrepareParameterStatement",
                                 &TestPreparedStatement::testPrepareParameterStatement));
  return suiteOfTests;
}
